{% extends "otree/BaseAdminRegular.html" %}

{% block head_title %}Create session{% endblock %}
{% block title %}Create session{% endblock %}

{% block content %}
<div class="alert alert-info">
    <p><b>Tip:</b> 
        consider creating your session in a <a href="{% url 'Rooms' %}">Room</a>,
        for more stable start links that are not opened accidentally.
    </p>
</div>    
{% include "otree/includes/CreateSessionForm.html" %}

{% endblock %}
