<nav class="navbar navbar-expand-sm navbar-dark bg-dark" id="top_menu">
    <div class="container-fluid">
        <a class="navbar-brand" href="/">oTree</a>

        <button class="navbar-toggler" type="button" data-bs-toggle="collapse" data-target="#navbarSupportedContent">
            <span class="navbar-toggler-icon"></span>
        </button>

        <div class="navbar-collapse collapse" id="navbarSupportedContent">
            <ul class="navbar-nav">
                <li class="nav-item"><a class="nav-link" href="{% url 'DemoIndex' %}">Demo</a></li>
                <li class="nav-item"><a class="nav-link" href="{% url 'Sessions' %}">Sessions</a></li>
                <li class="nav-item"><a class="nav-link" href="{% url 'Rooms' %}">Rooms</a></li>
                <li class="nav-item"><a class="nav-link" href="{% url 'ExportIndex' %}">Data</a></li>
                <li class="nav-item"><a class="nav-link" href="{% url 'ServerCheck' %}">Server Check</a></li>
                {% if view._is_logged_in() %}
                  <li class="nav-item"><a class="nav-link" href="{% url 'Logout' %}">Logout</a></li>
                {% endif %}
            </ul>
        </div>
    </div>
</nav>
